from urllib.parse import urljoin, urlparse
from warnings import warn

import requests
from bs4 import BeautifulSoup as BS


class MissingURLSchemaWarning(UserWarning):
    pass


class Clawler:
    def __init__(self, args):
        """Init."""
        self.urls = self._normalize_url(args.url)
        self.only_target = args.only_target
        self.level = args.level
        self.queue = []
        self.UA = "Mozilla/5.0 (Windows NT 5.1; rv:40.0) " "Gecko/20100101 Firefox/40.0"

    def run_crawl(self):
        """Execute clawler."""
        if self.only_target:
            return [set(self.urls)]
        for now_level in range(self.level):
            self._crawl(now_level)

        return self.queue

    def _crawl(self, now_level):
        """Helper for clawling."""
        collecting_links = set()
        collected_links = set().union(*self.queue)
        if now_level == 0:
            self.queue.append(set(self.urls))
        for url in self.queue[-1]:
            source = requests.get(url, headers={"User-Agent": self.UA}).content
            data = BS(source, parser="html.parser", features="lxml")
            extracted_url = [
                urljoin(url, _.get("href")) for _ in set(data.find_all("a"))
            ]
            collecting_links |= set(self._normalize_url(extracted_url))
        self.queue.append(collecting_links - collected_links)

    def _normalize_url(self, urls):
        """Normalize url."""
        valid_urls = []
        for url in urls:
            parsed_url = urlparse(url)
            if self._check_schema_is_invalid(parsed_url):
                warn(
                    "{}: schema {} is not valid.".format(url, parsed_url.scheme),
                    MissingURLSchemaWarning,
                )
            else:
                valid_urls.append(parsed_url.geturl())
        return valid_urls

    @staticmethod
    def _check_schema_is_invalid(parsed_url):
        """Judge if given url has a valid schema."""
        is_invalid = parsed_url.scheme not in ("http", "https", "ftp", "file")
        return parsed_url.scheme == "" or is_invalid



#!/usr/bin/env python
import argparse
import http.client as httplib
import sys
import textwrap

from wbsv import __version__
from wbsv.archiver import Archiver
from wbsv.crawler import Clawler


class HttpConnectionNotFountError(Exception):
    pass


def check_connectivity(url="www.google.com", timeout=3):
    conn = httplib.HTTPConnection(url, timeout=timeout)
    try:
        conn.request("HEAD", "/")
        conn.close()
        return True
    except Exception as e:
        print(e, file=sys.stderr)
        return False


def check_natural(v):
    if int(v) < 0:
        raise argparse.ArgumentTypeError("%s is an invalid natural number" % int(v))
    return int(v)


def check_positive(v):
    if int(v) <= 0:
        raise argparse.ArgumentTypeError("%s is an invalid natural number" % int(v))
    return int(v)


def parse_args(test=None):
    """Parse arguments."""
    parser = argparse.ArgumentParser(
        prog="wbsv",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=textwrap.dedent(
            """\
            CLI tool for save webpage on Wayback Machine forever.
            Save webpage and one 's all URI(s) on Wayback Machine."""
        ),
        epilog=textwrap.dedent(
            """\
            If you don't give the URL,
            interactive mode will be launched.
            (To quit interactive mode,
            type "end", "exit", "exit()",
            "break", "bye", ":q" or "finish".)"""
        ),
    )

    parser.add_argument(
        "url", metavar="url", nargs="*", type=str, help="Saving pages in order."
    )
    parser.add_argument(
        "-r",
        "--retry",
        type=check_natural,
        metavar="times",
        default=0,
        help="Set a retry limit on failed save.(>=0",
    )
    parser.add_argument(
        "-t", "--only_target", action="store_true", help="Save just target webpage(s)."
    )
    parser.add_argument(
        "-l",
        "--level",
        type=check_positive,
        metavar="level",
        default=1,
        help="Set maximum recursion depth. (>0)",
    )
    parser.add_argument(
        "-V", "--version", action="version", version="%(prog)s {}".format(__version__)
    )
    if test:
        return parser.parse_args(test)
    else:
        return parser.parse_args()


def usual(args):
    past, now, fail = 0, 0, 0
    print("[+]Target: {}".format(args.url))
    c = Clawler(args)
    retrieved_links = set().union(*c.run_crawl())
    len_links = len(retrieved_links)
    print("[+]{} URI(s) found.".format(len_links))
    a = Archiver(args)
    for ind, link in enumerate(retrieved_links, 1):
        print("[{:02d}/{}]: Wait...".format(ind, len_links), end="\r")
        archive = a.archive(link)
        if archive:
            archived_link, cached_flag = archive
            inc = cache_or_now(ind, len_links, archived_link, cached_flag)
            past += inc[0]
            now += inc[1]
        else:
            print("[{:02d}/{}]: <FAIL> {}".format(ind, len_links, link))
            fail += 1

    print("[+]FIN!: {}".format(args.url))
    print("[+]ALL: {}, NOW: {}, PAST: {}, FAIL: {}".format(len_links, now, past, fail))


def cache_or_now(ind, len_links, archived_link, cached_flag):
    if cached_flag:
        print("[{:02d}/{}]: <PAST> {}".format(ind, len_links, archived_link))
        return (1, 0)
    else:
        print("[{:02d}/{}]: <NOW> {}".format(ind, len_links, archived_link))
        return (0, 1)


def repl(args):
    finish_words = ["end", "exit", "exit()", "break", "bye", ":q", "finish"]
    print("[[Input a target url (ex: https://google.com)]]")
    while True:
        link = input(">>> ").rstrip()
        if link in finish_words:
            print("[+]End.")
            break
        elif link == "":
            pass
        else:
            args.url = [link]
            try:
                usual(args)
            except Exception as e:
                print(e, file=sys.stderr)


def main():
    if not check_connectivity():
        raise HttpConnectionNotFountError
    args = parse_args()
    if len(sys.argv) <= 1:
        repl(args)
    else:
        usual(args)


if __name__ == "__main__":
    main()



import sys

import waybackpy


class SavepagenowFailureError(Exception):
    pass


class Archiver:
    def __init__(self, args):
        """Init."""
        self.retry = args.retry
        self.UA = "Mozilla/5.0 (Windows NT 5.1; rv:40.0) " "Gecko/20100101 Firefox/40.0"

    def archive(self, url):
        """Archive link."""
        wp = waybackpy.Url(url, self.UA)
        for _ in range(self.retry + 1):
            if not self._try_savepagenow(wp):
                continue
            else:
                return (wp.archive_url, wp.cached_save)
        return False

    @staticmethod
    def _try_savepagenow(wp):
        """Error handler for saving with savepagenow."""
        try:
            wp.save()
        except waybackpy.exceptions.RedirectSaveError as e:
            print(e, file=sys.stderr)
            return False
        except waybackpy.exceptions.WaybackError as e:
            print(e, file=sys.stderr)
            return False
        except AttributeError as e:
            print(e, file=sys.stderr)
            return False
        else:
            return True



from setuptools import find_packages, setup

from wbsv import __version__

setup(
    name="wbsv",
    version=__version__,
    description="Throw all URIs in a page on to Wayback Machine from CLI.",
    description_content_type="",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/eggplants/wbsv-cli",
    author="eggplants",
    packages=find_packages(),
    python_requires=">=3.5",
    include_package_data=True,
    license="MIT",
    install_requires=open("requirements.txt").read().splitlines(),
    entry_points={"console_scripts": ["wbsv=wbsv.main:main"]},
)
